package com.me.asteroids.systems;

import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.me.asteroids.Utils;
import com.me.asteroids.components.AccelerationComponent;
import com.me.asteroids.components.ModelComponent;
import com.me.asteroids.components.PositionComponent;
import com.me.asteroids.components.VelocityComponent;
import com.me.asteroids.components.model.Model;
import com.me.common.ecs.ComponentMapper;
import com.me.common.ecs.Engine;
import com.me.common.ecs.Entity;
import com.me.common.ecs.EntitySystem;

public class MovementSystem extends EntitySystem {

    private Vector2 tmp = new Vector2();

    private ComponentMapper<PositionComponent> positionMapper;
    private ComponentMapper<VelocityComponent> velocityMapper;
    private ComponentMapper<AccelerationComponent> accelMapper;
    private ComponentMapper<ModelComponent> modelMapper;

    public MovementSystem(Engine engine) {
        super(engine, PositionComponent.class, VelocityComponent.class);

        positionMapper = engine.getComponentMapper(PositionComponent.class);
        velocityMapper = engine.getComponentMapper(VelocityComponent.class);
        accelMapper = engine.getComponentMapper(AccelerationComponent.class);
        modelMapper = engine.getComponentMapper(ModelComponent.class);
    }

    @Override
    public void processEntity(Entity entity, float dt) {
        PositionComponent positionComponent = positionMapper.get(entity);
        VelocityComponent velocityComponent = velocityMapper.get(entity);

        Vector2 velocity = velocityComponent.velocity;

        AccelerationComponent accelComponent = accelMapper.get(entity);
        if (accelComponent != null && !accelComponent.acceleration.isZero()) {
            velocity.add(tmp.set(accelComponent.acceleration).scl(dt));
        }

        float maxVelocity = velocityComponent.maxVelocity;
        Vector2 position = positionComponent.position;
        if (!velocity.isZero()) {
            velocity.clamp(0, maxVelocity);
            position.add(tmp.set(velocity).scl(dt));
        }

        float angularVelocity = velocityComponent.angularVelocity;
        if (!MathUtils.isZero(angularVelocity)) {
            positionComponent.rotation = Utils.wrapAngle(positionComponent.rotation + (angularVelocity * dt));
        }

        ModelComponent modelComponent = modelMapper.get(entity);
        if (modelComponent != null) {
            Model model = modelComponent.model;
            model.setPosition(position);
            model.setRotation(positionComponent.rotation - 90);
        }
    }

}
